Package: ggpp
Type: Package
Title: Grammar Extensions to 'ggplot2'
Version: 0.6.1
Date: 2026-07-01
Authors@R: 
    c(
    person("Pedro J.", "Aphalo", email = "pedro.aphalo@helsinki.fi", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3385-972X")),
    person("Kamil", "Slowikowski", role = "ctb", comment = c(ORCID = "0000-0002-2843-6370")),
    person("Michał", "Krassowski", role = "ctb", comment = c(ORCID = "0000-0002-9638-7785")),
    person("Daniel", "Sabanés Bové", role = "ctb", comment = c(ORCID = "0000-0002-0176-9239")),
    person("Stella", "Banjo", role = "ctb")
    )
Maintainer: Pedro J. Aphalo <pedro.aphalo@helsinki.fi>
Description: Extensions to 'ggplot2' respecting the grammar of graphics 
    paradigm. Geometries: geom_table(), geom_plot() and geom_grob() add insets to 
    plots using native data coordinates, while geom_table_npc(), geom_plot_npc()
    and geom_grob_npc() do the same using "npc" coordinates through new 
    aesthetics "npcx" and "npcy". Statistics: select observations based on 2D 
    density. Positions: radial nudging away from a center point and nudging away
    from a line or curve; combined stacking and nudging; combined dodging and
    nudging.
License: GPL (>= 2)
LazyData: true
LazyLoad: true
ByteCompile: true
Depends: R (>= 4.1.0), ggplot2 (>= 3.5.0)
Imports: stats, grid, grDevices, rlang (>= 1.0.6), vctrs (>= 0.6.0),
        glue (>= 1.6.0), gridExtra (>= 2.3), scales (>= 1.3.0), tibble
        (>= 3.1.8), dplyr (>= 1.1.0), xts (>= 0.13.0), zoo (>= 1.8-11),
        MASS (>= 7.3-58), polynom (>= 1.4-0), lubridate (>= 1.9.0),
        stringr (>= 1.4.0)
Suggests: knitr (>= 1.40), rmarkdown (>= 2.20), ggrepel (>= 0.9.2),
        gginnards(>= 0.2.0), magick (>= 2.7.3), testthat (>= 3.1.5),
        vdiffr (>= 1.0.5)
URL: https://docs.r4photobiology.info/ggpp/,
        https://github.com/aphalo/ggpp
BugReports: https://github.com/aphalo/ggpp/issues
Encoding: UTF-8
VignetteBuilder: knitr
Collate: 'annotate.r' 'compute-npc.r' 'dark-or-light.R'
        'example-data.R' 'geom-grob.R' 'ggpp-legend-draw.R'
        'utilities.R' 'ggp2-margins.R' 'geom-label-linked.r'
        'geom-label-npc.r' 'geom-label-pairwise.r'
        'geom-margin-arrow.r' 'geom-margin-grob.r'
        'geom-margin-point.r' 'geom-plot.R' 'geom-point-linked.r'
        'geom-quadrant-lines.R' 'geom-table.R' 'geom-text-linked.r'
        'geom-text-npc.r' 'geom-text-pairwise.R' 'ggpp.R' 'onload.R'
        'position-dodge-nudge-to.R' 'position-dodge-nudge.R'
        'position-dodge2-nudge.R' 'position-dodge2nudge-to.R'
        'position-jitter-nudge.R' 'position-nudge-center.R'
        'position-nudge-line.R' 'position-stack-nudge.R'
        'position-stacknudge-to.R' 'scale-continuous-npc.r'
        'stat-apply.R' 'stat-dens1d-filter.r' 'stat-dens1d-labels.r'
        'stat-dens2d-filter.r' 'stat-dens2d-labels.r'
        'stat-format-table.R' 'stat-functions.R' 'stat-panel-counts.R'
        'stat-quadrant-counts.R' 'try-data-frame.R' 'weather-data.R'
        'wrap-labels.R'
Config/Needs/website: rmarkdown
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-01 22:30:55 UTC; aphalo
Author: Pedro J. Aphalo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3385-972X>),
  Kamil Slowikowski [ctb] (ORCID:
    <https://orcid.org/0000-0002-2843-6370>),
  Michał Krassowski [ctb] (ORCID:
    <https://orcid.org/0000-0002-9638-7785>),
  Daniel Sabanés Bové [ctb] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Stella Banjo [ctb]
Repository: CRAN
Date/Publication: 2026-07-02 22:10:10 UTC
Built: R 4.6.1; ; 2026-07-02 22:42:22 UTC; unix
