# Generated by roxygen2: do not edit by hand

S3method(ggplot,ts)
S3method(ggplot,xts)
S3method(heightDetails,titleGrob)
S3method(widthDetails,titleGrob)
export(GeomGrob)
export(GeomGrobNpc)
export(GeomLabelNpc)
export(GeomLabelPairwise)
export(GeomLabelS)
export(GeomPlot)
export(GeomPlotNpc)
export(GeomPointS)
export(GeomQuadrantLines)
export(GeomTable)
export(GeomTableNpc)
export(GeomTextNpc)
export(GeomTextPairwise)
export(GeomTextS)
export(GeomVHLines)
export(GeomXMarginArrow)
export(GeomXMarginGrob)
export(GeomXMarginPoint)
export(GeomYMarginArrow)
export(GeomYMarginGrob)
export(GeomYMarginPoint)
export(PositionDodge2AndNudgeTo)
export(PositionDodgeNudgeTo)
export(PositionFillAndNudge)
export(PositionFillNudgeTo)
export(PositionNudgeCenter)
export(PositionNudgeLine)
export(PositionStackNudgeTo)
export(StatApplyGroup)
export(StatDens1dFilter)
export(StatDens1dFilterG)
export(StatDens1dLabels)
export(StatDens2dFilter)
export(StatDens2dFilterG)
export(StatDens2dLabels)
export(StatFmtTb)
export(StatFunctions)
export(StatGroupCounts)
export(StatPanelCounts)
export(StatQuadrantCounts)
export(annotate)
export(as_npc)
export(as_npcx)
export(as_npcy)
export(compute_npc)
export(compute_npcx)
export(compute_npcy)
export(dark_or_light)
export(draw_key_label_s)
export(draw_key_text_s)
export(geom_grob)
export(geom_grob_npc)
export(geom_label_npc)
export(geom_label_pairwise)
export(geom_label_s)
export(geom_plot)
export(geom_plot_npc)
export(geom_point_s)
export(geom_quadrant_lines)
export(geom_table)
export(geom_table_npc)
export(geom_text_npc)
export(geom_text_pairwise)
export(geom_text_s)
export(geom_vhlines)
export(geom_x_margin_arrow)
export(geom_x_margin_grob)
export(geom_x_margin_point)
export(geom_y_margin_arrow)
export(geom_y_margin_grob)
export(geom_y_margin_point)
export(position_dodge2_keep)
export(position_dodge2nudge)
export(position_dodge2nudge_to)
export(position_dodge_keep)
export(position_dodgenudge)
export(position_dodgenudge_to)
export(position_fill_keep)
export(position_fillnudge)
export(position_fillnudge_to)
export(position_jitter_keep)
export(position_jitternudge)
export(position_nudge_center)
export(position_nudge_centre)
export(position_nudge_keep)
export(position_nudge_line)
export(position_nudge_to)
export(position_stack_keep)
export(position_stack_minmax)
export(position_stacknudge)
export(position_stacknudge_to)
export(scale_npcx_continuous)
export(scale_npcy_continuous)
export(stat_apply_group)
export(stat_centroid)
export(stat_dens1d_filter)
export(stat_dens1d_filter_g)
export(stat_dens1d_labels)
export(stat_dens2d_filter)
export(stat_dens2d_filter_g)
export(stat_dens2d_labels)
export(stat_fmt_tb)
export(stat_functions)
export(stat_group_counts)
export(stat_panel_counts)
export(stat_quadrant_counts)
export(stat_summary_xy)
export(try_data_frame)
export(try_tibble)
export(ttheme_gtbw)
export(ttheme_gtdark)
export(ttheme_gtdefault)
export(ttheme_gtlight)
export(ttheme_gtminimal)
export(ttheme_gtplain)
export(ttheme_gtsimple)
export(ttheme_gtstripes)
export(ttheme_set)
export(wrap_labels)
import(ggplot2)
import(grid)
import(scales)
importFrom(MASS,kde2d)
importFrom(dplyr,mutate)
importFrom(ggplot2,ggplot)
importFrom(polynom,polynomial)
importFrom(rlang,.data)
